#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = __importStar(require("react"));
const ink_1 = require("ink");
const ink_big_text_1 = __importDefault(require("ink-big-text"));
const repl_ns_1 = require("repl-ns");
const ink_form_1 = require("ink-form");
const ink_spinner_1 = __importDefault(require("ink-spinner"));
const ytdl_core_muxer_1 = __importDefault(require("ytdl-core-muxer"));
const jotai_1 = require("jotai");
const ffmpeg_static_1 = __importDefault(require("ffmpeg-static"));
const fluent_ffmpeg_1 = __importDefault(require("fluent-ffmpeg"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const os_1 = require("os");
if (ffmpeg_static_1.default)
    process.env['FFMPEG_PATH'] = ffmpeg_static_1.default;
const mainNS = (0, repl_ns_1.ns)('main', {
    Main: () => {
        const [url, setUrl] = (0, jotai_1.useAtom)(mainNS().url$);
        const [path, setPath] = (0, jotai_1.useAtom)(mainNS().path$);
        const [startTime, setStartTime] = (0, jotai_1.useAtom)(mainNS().startTime$);
        const [endTime, setEndTime] = (0, jotai_1.useAtom)(mainNS().endTime$);
        const [downloaded, setDownloaded] = (0, jotai_1.useAtom)(mainNS().downloaded$);
        const [downloadInProgress, setDownloadInProgress] = (0, jotai_1.useAtom)(mainNS().downloadInProgress$);
        const [downloadError, setDownloadError] = (0, jotai_1.useAtom)(mainNS().downloadError$);
        return (React.createElement(ink_1.Box, { flexDirection: "column" },
            React.createElement(ink_1.Box, null,
                React.createElement(ink_big_text_1.default, { text: "YOUTUBE DOWNLOAD" })),
            React.createElement(ink_1.Box, { marginBottom: 1 }, downloadInProgress ? (React.createElement(ink_spinner_1.default, { type: "aesthetic" })) : downloaded ? (React.createElement(ink_1.Text, { color: 'green' }, "\u2705 Video downloaded!")) : downloadError ? (React.createElement(ink_1.Text, { color: 'red' }, downloadError.message)) : null),
            React.createElement(ink_1.Box, null,
                React.createElement(ink_form_1.Form, { ...mainNS().formProps({ url, path, startTime, endTime }, { setUrl, setPath, setStartTime, setEndTime }), onSubmit: obj => {
                        setDownloaded(() => false);
                        setDownloadInProgress(() => true);
                        mainNS().onDownload(obj, { setDownloaded, setDownloadInProgress, setDownloadError });
                    } }))));
    },
    formProps: ({ url, path, startTime, endTime }, { setUrl, setPath, setStartTime, setEndTime, }) => ({
        onChange: (state) => {
            setUrl(() => state.url);
            setPath(() => state.path);
            setStartTime(() => state.startTime);
            setEndTime(() => state.endTime);
        },
        form: {
            title: 'Please setup form for downloading video from Youtube',
            sections: [
                {
                    title: '',
                    fields: [
                        {
                            type: 'string',
                            name: 'url',
                            label: 'Youtube URL',
                            regex: RegExp('^https://.*$'),
                            initialValue: url,
                        },
                        {
                            type: 'string',
                            name: 'path',
                            label: 'File location',
                            initialValue: path || `${process.cwd()}/video.mp4`,
                        },
                        {
                            type: 'string',
                            name: 'startTime',
                            label: 'Start time (optional)',
                            regex: RegExp('(^\\d\\d:\\d\\d:\\d\\d$|^$)'),
                            initialValue: startTime,
                        },
                        {
                            type: 'string',
                            name: 'endTime',
                            label: 'End time (optional)',
                            regex: RegExp('(^\\d\\d:\\d\\d:\\d\\d$|^$)'),
                            initialValue: endTime,
                        },
                    ],
                },
            ],
        },
    }),
    async onDownload(obj, { setDownloaded, setDownloadInProgress, setDownloadError, }) {
        const renderComplete = () => {
            setDownloadInProgress(() => false);
            setDownloaded(() => true);
            setDownloadError(() => null);
        };
        const renderErr = (err) => {
            setDownloadInProgress(() => false);
            setDownloaded(() => false);
            setDownloadError(() => err);
        };
        try {
            const temporaryFilePath = `${await (0, promises_1.mkdtemp)(`${(0, os_1.tmpdir)()}/ytdl-tui-`)}/video.mp4`;
            (0, ytdl_core_muxer_1.default)(obj.url)
                .on('error', err => renderErr(err))
                .pipe(fs_1.default.createWriteStream(temporaryFilePath).on('error', err => renderErr(err)))
                .on('finish', async () => {
                try {
                    if (obj.startTime || obj.endTime) {
                        let ffmpegStream = (0, fluent_ffmpeg_1.default)(temporaryFilePath);
                        if (obj.startTime) {
                            ffmpegStream = ffmpegStream.setStartTime(obj.startTime);
                        }
                        if (obj.endTime) {
                            const startSec = obj.startTime
                                ? (() => {
                                    const [hh = 0, mm = 0, ss = 0] = obj.startTime.split(':').map(Number);
                                    return ss + mm * 60 + hh * 60 * 60;
                                })()
                                : 0;
                            const endSec = (() => {
                                const [hh = 0, mm = 0, ss = 0] = obj.endTime.split(':').map(Number);
                                return ss + mm * 60 + hh * 60 * 60;
                            })();
                            if (endSec - startSec <= 0) {
                                throw new Error("End time can't be less than start time");
                            }
                            ffmpegStream = ffmpegStream.setDuration(endSec - startSec);
                        }
                        ffmpegStream
                            .saveToFile(obj.path)
                            .on('error', err => renderErr(err))
                            .on('end', () => renderComplete());
                    }
                    else {
                        await (0, promises_1.copyFile)(temporaryFilePath, obj.path);
                        renderComplete();
                    }
                }
                catch (err) {
                    renderErr(err);
                }
            });
        }
        catch (err) {
            renderErr(err);
        }
    },
    url$: (0, jotai_1.atom)(''),
    path$: (0, jotai_1.atom)(''),
    startTime$: (0, jotai_1.atom)(''),
    endTime$: (0, jotai_1.atom)(''),
    downloaded$: (0, jotai_1.atom)(false),
    downloadInProgress$: (0, jotai_1.atom)(false),
    downloadError$: (0, jotai_1.atom)(null),
});
(async () => {
    await mainNS.ready;
    const Main = mainNS().Main;
    (0, ink_1.render)(React.createElement(Main, null));
})();
